#ifndef __ASM_S390_BITSPERLONG_H
#define __ASM_S390_BITSPERLONG_H

#ifndef __s390x__
#define __BITS_PER_LONG 32
#else
#define __BITS_PER_LONG 64
#endif

#include <asm-generic/bitsperlong.h>

#endif /* __ASM_S390_BITSPERLONG_H */

