/*
 *  Copyright (C) 2008 Ilya Yanok, Emcraft Systems
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#ifndef SOCRATES_FPGA_PIC_H
#define SOCRATES_FPGA_PIC_H

void socrates_fpga_pic_init(struct device_node *pic);

#endif
